import { MigrationInterface, QueryRunner } from 'typeorm';

export class InitialSchema1700000000001 implements MigrationInterface {
  name = 'InitialSchema1700000000001';

  public async up(queryRunner: QueryRunner): Promise<void> {
    // Create users table
    await queryRunner.query(`
      CREATE TABLE "users" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "email" character varying NOT NULL,
        "username" character varying NOT NULL,
        "password" character varying NOT NULL,
        "avatar" character varying,
        "bio" text,
        "isOnline" boolean NOT NULL DEFAULT true,
        "lastSeen" timestamp,
        "publicKey" text,
        "privateKey" text,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        "updatedAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "UQ_users_email" UNIQUE ("email"),
        CONSTRAINT "PK_users" PRIMARY KEY ("id")
      )
    `);

    // Create organizations table
    await queryRunner.query(`
      CREATE TABLE "organizations" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "name" character varying NOT NULL,
        "slug" character varying NOT NULL,
        "description" text,
        "logo" character varying,
        "website" character varying,
        "contactEmail" character varying,
        "plan" character varying NOT NULL DEFAULT 'free',
        "status" character varying NOT NULL DEFAULT 'active',
        "settings" jsonb,
        "limits" jsonb,
        "subscriptionExpiry" timestamp,
        "ownerId" uuid,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        "updatedAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "UQ_organizations_name" UNIQUE ("name"),
        CONSTRAINT "UQ_organizations_slug" UNIQUE ("slug"),
        CONSTRAINT "PK_organizations" PRIMARY KEY ("id")
      )
    `);

    // Create organization_members table
    await queryRunner.query(`
      CREATE TABLE "organization_members" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "role" character varying NOT NULL DEFAULT 'member',
        "status" character varying NOT NULL DEFAULT 'active',
        "permissions" jsonb,
        "joinedAt" timestamp,
        "lastActiveAt" timestamp,
        "organizationId" uuid NOT NULL,
        "userId" uuid NOT NULL,
        "invitedById" uuid,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        "updatedAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "UQ_organization_members_org_user" UNIQUE ("organizationId", "userId"),
        CONSTRAINT "PK_organization_members" PRIMARY KEY ("id")
      )
    `);

    // Create organization_invites table
    await queryRunner.query(`
      CREATE TABLE "organization_invites" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "email" character varying NOT NULL,
        "role" character varying NOT NULL DEFAULT 'member',
        "status" character varying NOT NULL DEFAULT 'pending',
        "token" character varying NOT NULL,
        "expiresAt" timestamp NOT NULL,
        "message" text,
        "organizationId" uuid NOT NULL,
        "invitedById" uuid NOT NULL,
        "acceptedById" uuid,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        "updatedAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "PK_organization_invites" PRIMARY KEY ("id")
      )
    `);

    // Create chat_rooms table
    await queryRunner.query(`
      CREATE TABLE "chat_rooms" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "name" character varying,
        "description" text,
        "type" character varying NOT NULL DEFAULT 'direct',
        "avatar" character varying,
        "isEncrypted" boolean NOT NULL DEFAULT false,
        "encryptionKey" text,
        "organizationId" uuid NOT NULL,
        "createdById" uuid,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        "updatedAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "PK_chat_rooms" PRIMARY KEY ("id")
      )
    `);

    // Create messages table
    await queryRunner.query(`
      CREATE TABLE "messages" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "content" text NOT NULL,
        "type" character varying NOT NULL DEFAULT 'text',
        "isEncrypted" boolean NOT NULL DEFAULT false,
        "encryptedContent" text,
        "isEdited" boolean NOT NULL DEFAULT false,
        "isDeleted" boolean NOT NULL DEFAULT false,
        "editedAt" timestamp,
        "replyToId" uuid,
        "senderId" uuid NOT NULL,
        "chatRoomId" uuid NOT NULL,
        "organizationId" uuid NOT NULL,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        "updatedAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "PK_messages" PRIMARY KEY ("id")
      )
    `);

    // Create message_media table
    await queryRunner.query(`
      CREATE TABLE "message_media" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "filename" character varying NOT NULL,
        "originalName" character varying NOT NULL,
        "mimeType" character varying NOT NULL,
        "size" integer NOT NULL,
        "url" character varying NOT NULL,
        "type" character varying NOT NULL DEFAULT 'other',
        "thumbnailUrl" character varying,
        "width" integer,
        "height" integer,
        "duration" integer,
        "messageId" uuid NOT NULL,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "PK_message_media" PRIMARY KEY ("id")
      )
    `);

    // Create message_mentions table
    await queryRunner.query(`
      CREATE TABLE "message_mentions" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "messageId" uuid NOT NULL,
        "mentionedUserId" uuid NOT NULL,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "PK_message_mentions" PRIMARY KEY ("id")
      )
    `);

    // Create user_devices table
    await queryRunner.query(`
      CREATE TABLE "user_devices" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "deviceId" character varying NOT NULL,
        "deviceType" character varying NOT NULL,
        "platform" character varying NOT NULL,
        "fcmToken" text,
        "isActive" boolean NOT NULL DEFAULT true,
        "userId" uuid NOT NULL,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        "updatedAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "PK_user_devices" PRIMARY KEY ("id")
      )
    `);

    // Create notifications table
    await queryRunner.query(`
      CREATE TABLE "notifications" (
        "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
        "type" character varying NOT NULL,
        "title" character varying NOT NULL,
        "body" text NOT NULL,
        "data" jsonb,
        "isRead" boolean NOT NULL DEFAULT false,
        "isSent" boolean NOT NULL DEFAULT false,
        "recipientId" uuid NOT NULL,
        "messageId" uuid,
        "organizationId" uuid NOT NULL,
        "createdAt" timestamp NOT NULL DEFAULT now(),
        CONSTRAINT "PK_notifications" PRIMARY KEY ("id")
      )
    `);

    // Create chat_room_members table (many-to-many)
    await queryRunner.query(`
      CREATE TABLE "chat_room_members" (
        "chatRoomId" uuid NOT NULL,
        "userId" uuid NOT NULL,
        CONSTRAINT "PK_chat_room_members" PRIMARY KEY ("chatRoomId", "userId")
      )
    `);

    // Add foreign key constraints
    await queryRunner.query(`
      ALTER TABLE "organizations" 
      ADD CONSTRAINT "FK_organizations_owner" 
      FOREIGN KEY ("ownerId") REFERENCES "users"("id") ON DELETE SET NULL
    `);

    await queryRunner.query(`
      ALTER TABLE "organization_members" 
      ADD CONSTRAINT "FK_organization_members_organization" 
      FOREIGN KEY ("organizationId") REFERENCES "organizations"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "organization_members" 
      ADD CONSTRAINT "FK_organization_members_user" 
      FOREIGN KEY ("userId") REFERENCES "users"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "organization_members" 
      ADD CONSTRAINT "FK_organization_members_invitedBy" 
      FOREIGN KEY ("invitedById") REFERENCES "users"("id") ON DELETE SET NULL
    `);

    await queryRunner.query(`
      ALTER TABLE "organization_invites" 
      ADD CONSTRAINT "FK_organization_invites_organization" 
      FOREIGN KEY ("organizationId") REFERENCES "organizations"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "organization_invites" 
      ADD CONSTRAINT "FK_organization_invites_invitedBy" 
      FOREIGN KEY ("invitedById") REFERENCES "users"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "organization_invites" 
      ADD CONSTRAINT "FK_organization_invites_acceptedBy" 
      FOREIGN KEY ("acceptedById") REFERENCES "users"("id") ON DELETE SET NULL
    `);

    await queryRunner.query(`
      ALTER TABLE "chat_rooms" 
      ADD CONSTRAINT "FK_chat_rooms_organization" 
      FOREIGN KEY ("organizationId") REFERENCES "organizations"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "chat_rooms" 
      ADD CONSTRAINT "FK_chat_rooms_createdBy" 
      FOREIGN KEY ("createdById") REFERENCES "users"("id") ON DELETE SET NULL
    `);

    await queryRunner.query(`
      ALTER TABLE "messages" 
      ADD CONSTRAINT "FK_messages_sender" 
      FOREIGN KEY ("senderId") REFERENCES "users"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "messages" 
      ADD CONSTRAINT "FK_messages_chatRoom" 
      FOREIGN KEY ("chatRoomId") REFERENCES "chat_rooms"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "messages" 
      ADD CONSTRAINT "FK_messages_organization" 
      FOREIGN KEY ("organizationId") REFERENCES "organizations"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "messages" 
      ADD CONSTRAINT "FK_messages_replyTo" 
      FOREIGN KEY ("replyToId") REFERENCES "messages"("id") ON DELETE SET NULL
    `);

    await queryRunner.query(`
      ALTER TABLE "message_media" 
      ADD CONSTRAINT "FK_message_media_message" 
      FOREIGN KEY ("messageId") REFERENCES "messages"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "message_mentions" 
      ADD CONSTRAINT "FK_message_mentions_message" 
      FOREIGN KEY ("messageId") REFERENCES "messages"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "message_mentions" 
      ADD CONSTRAINT "FK_message_mentions_mentionedUser" 
      FOREIGN KEY ("mentionedUserId") REFERENCES "users"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "user_devices" 
      ADD CONSTRAINT "FK_user_devices_user" 
      FOREIGN KEY ("userId") REFERENCES "users"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "notifications" 
      ADD CONSTRAINT "FK_notifications_recipient" 
      FOREIGN KEY ("recipientId") REFERENCES "users"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "notifications" 
      ADD CONSTRAINT "FK_notifications_message" 
      FOREIGN KEY ("messageId") REFERENCES "messages"("id") ON DELETE SET NULL
    `);

    await queryRunner.query(`
      ALTER TABLE "notifications" 
      ADD CONSTRAINT "FK_notifications_organization" 
      FOREIGN KEY ("organizationId") REFERENCES "organizations"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "chat_room_members" 
      ADD CONSTRAINT "FK_chat_room_members_chatRoom" 
      FOREIGN KEY ("chatRoomId") REFERENCES "chat_rooms"("id") ON DELETE CASCADE
    `);

    await queryRunner.query(`
      ALTER TABLE "chat_room_members" 
      ADD CONSTRAINT "FK_chat_room_members_user" 
      FOREIGN KEY ("userId") REFERENCES "users"("id") ON DELETE CASCADE
    `);

    // Create indexes for better performance
    await queryRunner.query(`CREATE INDEX "IDX_users_email" ON "users" ("email")`);
    await queryRunner.query(`CREATE INDEX "IDX_organizations_slug" ON "organizations" ("slug")`);
    await queryRunner.query(`CREATE INDEX "IDX_organization_members_org" ON "organization_members" ("organizationId")`);
    await queryRunner.query(`CREATE INDEX "IDX_organization_members_user" ON "organization_members" ("userId")`);
    await queryRunner.query(`CREATE INDEX "IDX_organization_invites_email" ON "organization_invites" ("email")`);
    await queryRunner.query(`CREATE INDEX "IDX_organization_invites_token" ON "organization_invites" ("token")`);
    await queryRunner.query(`CREATE INDEX "IDX_chat_rooms_organization" ON "chat_rooms" ("organizationId")`);
    await queryRunner.query(`CREATE INDEX "IDX_messages_chatRoom" ON "messages" ("chatRoomId")`);
    await queryRunner.query(`CREATE INDEX "IDX_messages_sender" ON "messages" ("senderId")`);
    await queryRunner.query(`CREATE INDEX "IDX_messages_organization" ON "messages" ("organizationId")`);
    await queryRunner.query(`CREATE INDEX "IDX_messages_createdAt" ON "messages" ("createdAt")`);
    await queryRunner.query(`CREATE INDEX "IDX_notifications_recipient" ON "notifications" ("recipientId")`);
    await queryRunner.query(`CREATE INDEX "IDX_notifications_organization" ON "notifications" ("organizationId")`);
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    // Drop indexes
    await queryRunner.query(`DROP INDEX "IDX_notifications_organization"`);
    await queryRunner.query(`DROP INDEX "IDX_notifications_recipient"`);
    await queryRunner.query(`DROP INDEX "IDX_messages_createdAt"`);
    await queryRunner.query(`DROP INDEX "IDX_messages_organization"`);
    await queryRunner.query(`DROP INDEX "IDX_messages_sender"`);
    await queryRunner.query(`DROP INDEX "IDX_messages_chatRoom"`);
    await queryRunner.query(`DROP INDEX "IDX_chat_rooms_organization"`);
    await queryRunner.query(`DROP INDEX "IDX_organization_invites_token"`);
    await queryRunner.query(`DROP INDEX "IDX_organization_invites_email"`);
    await queryRunner.query(`DROP INDEX "IDX_organization_members_user"`);
    await queryRunner.query(`DROP INDEX "IDX_organization_members_org"`);
    await queryRunner.query(`DROP INDEX "IDX_organizations_slug"`);
    await queryRunner.query(`DROP INDEX "IDX_users_email"`);

    // Drop tables in reverse order
    await queryRunner.query(`DROP TABLE "chat_room_members"`);
    await queryRunner.query(`DROP TABLE "notifications"`);
    await queryRunner.query(`DROP TABLE "user_devices"`);
    await queryRunner.query(`DROP TABLE "message_mentions"`);
    await queryRunner.query(`DROP TABLE "message_media"`);
    await queryRunner.query(`DROP TABLE "messages"`);
    await queryRunner.query(`DROP TABLE "chat_rooms"`);
    await queryRunner.query(`DROP TABLE "organization_invites"`);
    await queryRunner.query(`DROP TABLE "organization_members"`);
    await queryRunner.query(`DROP TABLE "organizations"`);
    await queryRunner.query(`DROP TABLE "users"`);
  }
}